﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections ;
using XDesigner.ORM.DOM;
using System.IO;

namespace XDesigner.ORM
{
    public class CodeGenerator
    {
        public static void GenareteCode2(string fileName, string outputFilename)
        {
            using (StreamReader reader = new StreamReader(fileName, true))
            {
                using (StreamWriter writer = new StreamWriter(outputFilename, false , Encoding.Default ))
                {
                    string strLine = reader.ReadLine();
                    while (strLine != null)
                    {
                        strLine = strLine.Replace("\\", "\\\\");
                        strLine = strLine.Replace("\"", "\\\"");
                        writer.WriteLine("writer.WriteLine(\"" + strLine + "\");");

                        strLine = reader.ReadLine();
                    }
                }
            }
        }

        private static string ToStringValueCode(string v)
        {
            if (v == null)
            {
                return "null";
            }
            else if (v.Length == 0)
            {
                return  "string.Empty";
            }
            else
            {
                StringBuilder str = new StringBuilder();
                str.Append("\"");
                foreach (char c in v)
                {
                    if (c == '\r')
                        str.Append(@"\r");
                    else if (c == '\n')
                        str.Append(@"\n");
                    else if (c == '\"')
                        str.Append("\\");
                    else
                        str.Append(c);
                }
                str.Append("\"");
                return str.ToString();
            }
        }

        internal static string ToTypeNameCode(Type type, bool fullName)
        {
            string result = null;
            if (type.IsGenericType)
            {
                Type baseType = type.GetGenericTypeDefinition();
                Type[] aTypes = type.GetGenericArguments();
                StringBuilder str = new StringBuilder();
                foreach (Type t in aTypes)
                {
                    if (str.Length > 0)
                    {
                        str.Append(",");
                    }
                    str.Append(ToTypeNameCode(t, true));
                }//foreach
                string strFullName = baseType.FullName;
                int index = strFullName.IndexOf("`");
                if (index > 0)
                {
                    strFullName = strFullName.Substring(0, index);
                    result = strFullName + "<" + str.ToString() + ">";
                }
                else
                {
                    result = type.FullName;
                }
            }
            else if (type.IsArray)
            {
                Type itemType = type.GetElementType();
                result = ToTypeNameCode(itemType, fullName) + "[]";
            }
            else
            {
                if (fullName)
                {
                    result = type.FullName;
                }
                else
                {
                    result = type.Name;
                }
            }
            result = result.Replace("+", ".");
            return result;
        }

        private static void WriteCommandCode(
            TextWriter writer,
            ORMDBCommand cmd,
            string variableName ,
            string instanceName ,
            string currentPropertyName )
        {
            //writer.WriteLine("        object parameterValue = null;");
            writer.WriteLine("        " + variableName + ".CommandText = " + ToStringValueCode(cmd.CommandText) + ";");
            foreach (ORMDBParameter p in cmd.Parameters)
            {
                ORMPropertyInfo prop = (ORMPropertyInfo)p.Value;
                ORMRelationInfo relation = prop.Relation;
                string valueCode = null;
                if (relation != null)
                {
                    if (relation.ChildProperty == null)
                    {
                        throw new NotSupportedException( 
                            string.Format( ORMStrings.NotSupportedDelayLoad_Name , prop.Name ));
                    }

                    if (string.Compare(prop.Name, currentPropertyName, true) == 0)
                    {
                        valueCode = "this.DBValue_" + prop.Name;
                    }
                    else
                    {
                        if (relation.RuntimeStyle == ORMRelationStyle.OneToOne)
                        {
                            //writer.WriteLine("      if( " + instanceName + "." + prop.Name + " == null )");
                            //writer.WriteLine("           parameterValue = null;");
                            //writer.WriteLine("      else");
                            //writer.WriteLine("           parameterValue = " + instanceName + "." + prop.Name + "." + relation.ChildProperty.Name + " ;");

                            valueCode = instanceName + "." + prop.Name
                                + " == null ? (object)DBNull.Value : ( object )" + instanceName + "."
                                + prop.Name + "." + relation.ChildProperty.Name;

                        }
                        else if (relation.RuntimeStyle == ORMRelationStyle.OneToManyWithKeyList)
                        {
                            valueCode = instanceName + "." + prop.Name
                                + " == null ? ( object) DBNull.Value : ( object )base.GetKeyValueList( " + instanceName + "."
                                + prop.Name + ", \"" + relation.ChildProperty.Name + "\")";
                        }
                    }
                }
                else
                {
                    valueCode = instanceName + "." + prop.Name ;
                }
                writer.WriteLine("        " + variableName + ".Parameters.AddParameter( "
                    + ToStringValueCode(p.ParameterName) + " , " + valueCode + " );");
            }//foreach
        }

        public static string GenerateClassesCode(
            ORMContext context, 
            ORMTypeInfo[] types, 
            string nameSpace ,
            List<Type> referenceTypes)
        {
            StringWriter writer = new StringWriter();
            writer.WriteLine("using System;");
            writer.WriteLine("using System.Data;");
            writer.WriteLine("using System.Collections.Generic;");
            writer.WriteLine("using System.Collections;");
            writer.WriteLine("using XDesigner.ORM;");
            writer.WriteLine("namespace " + nameSpace);
            writer.WriteLine("{");
            foreach (ORMTypeInfo type in types)
            {
                writer.WriteLine("");
                writer.WriteLine(""); 
                writer.WriteLine(GenerateClassCode(context, type, referenceTypes));
            }
            writer.WriteLine("}");
            return writer.ToString();
        }


        public static string GenerateClassCode(ORMContext context, ORMTypeInfo type , List<Type> referenceTypes )
        {
            ReflectionORMTypeMappingHelper helper = new ReflectionORMTypeMappingHelper( type );
            if ( referenceTypes != null 
                && referenceTypes.Contains(type.InstanceType) == false)
            {
                referenceTypes.Add(type.InstanceType);
            }
            string fullTypeName = ToTypeNameCode( type.InstanceType , true );
            string typeName = ToTypeNameCode( type.InstanceType , false );

            bool generateTempClass = true;
            foreach (ORMPropertyInfo p in type.Properties)
            {
                if (p.DelayLoad)
                {
                    generateTempClass = true;
                    break;
                }
            }
            //string tempClassName = typeName + "_Temp";
            
            System.IO.StringWriter writer = new System.IO.StringWriter();
            
            writer.WriteLine("public class " + typeName +"ORMTypeMappingHelper : " + typeof( ORMTypeMappingHelper).FullName );
            writer.WriteLine("{");
            writer.WriteLine("    public " + typeName +"ORMTypeMappingHelper()");
            writer.WriteLine("    {");
            writer.WriteLine("");
            writer.WriteLine("    }");
            writer.WriteLine("");
            writer.WriteLine("    public override bool SupportDelayLoad");
            writer.WriteLine("    {");
            writer.WriteLine("        get");
            writer.WriteLine("        {");
            writer.WriteLine("            return true;");
            writer.WriteLine("        }");
            writer.WriteLine("    }");
            writer.WriteLine("");
            writer.WriteLine("    public override Type InstanceType");
            writer.WriteLine("    {");
            writer.WriteLine("        get");
            writer.WriteLine("        {");
            if (generateTempClass)
            {
                writer.WriteLine("            return typeof(" + typeName + "_Temp);");
            }
            else
            {
                writer.WriteLine("            return typeof(" + fullTypeName + " );");
            }
            writer.WriteLine("        }");
            writer.WriteLine("    }");
            writer.WriteLine("");
            writer.WriteLine("    public override object CreateInstance()");
            writer.WriteLine("    {");
            if (generateTempClass)
            {
                writer.WriteLine("        return new " + typeName + "_Temp();");
            }
            else
            {
                writer.WriteLine("        return new " + fullTypeName + "();");
            }
            writer.WriteLine("    }");
            writer.WriteLine("");
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("");
            writer.WriteLine("    public override bool FillInstance(ORMContext context, IDataReader reader, object instance, List<RelationPropertyValue> relationValues)");
            writer.WriteLine("    {");
            writer.WriteLine("        " + fullTypeName + " obj = (" + fullTypeName + ")instance;");
            if (generateTempClass)
            {
                writer.WriteLine("        if (instance is " + typeName + "_Temp )");
                writer.WriteLine("        {");
                writer.WriteLine("            " + typeName + "_Temp temp = (" + typeName + "_Temp )instance;");
                writer.WriteLine("            temp._Work = context.Framework;");
                writer.WriteLine("        }");
            }
            writer.WriteLine("");
            writer.WriteLine("        int fieldCount = reader.FieldCount;");
            writer.WriteLine("        bool enableDelayLoad = context.Framework.Config.EnableDelayLoad;");
            writer.WriteLine("        for (int iCount = 0; iCount < fieldCount; iCount++)");
            writer.WriteLine("        {");
            writer.WriteLine("            string name = reader.GetName(iCount);");
            writer.WriteLine("            name = name.Trim().ToLower();");
            writer.WriteLine("            switch (name)");
            writer.WriteLine("            {");
            Hashtable fieldMap = new Hashtable();
            foreach (ORMPropertyInfo property in type.Properties)
            {
                string fieldName = property.RuntimeDBFieldName;
                if (Util.HasContent(fieldName))
                {
                    ArrayList list = (ArrayList)fieldMap[fieldName];
                    if (list == null)
                    {
                        list = new ArrayList();
                        fieldMap[fieldName] = list;
                    }
                    list.Add(property);
                }
                fieldName = property.ComparabilityDBFieldName;
                if (Util.HasContent(fieldName))
                {
                    ArrayList list = (ArrayList)fieldMap[fieldName];
                    if (list == null)
                    {
                        list = new ArrayList();
                        fieldMap[fieldName] = list;
                    }
                    list.Add(property);
                }

                fieldName = property.ComparabilityDBFieldName2;
                if (Util.HasContent(fieldName))
                {
                    ArrayList list = (ArrayList)fieldMap[fieldName];
                    if (list == null)
                    {
                        list = new ArrayList();
                        fieldMap[fieldName] = list;
                    }
                    list.Add(property);
                }
            }
            foreach (string fieldName in fieldMap.Keys)
            {
                writer.WriteLine("                case \"" + fieldName.Trim().ToLower() + "\":");
                writer.WriteLine("                {");
                ArrayList ps = (ArrayList)fieldMap[fieldName];
                foreach (ORMPropertyInfo property in ps)
                {
                    if (property.HasRelation)
                    {
                        if (property.DelayLoad)
                        {
                            ORMRelationInfo relation = property.Relation;
                            writer.WriteLine("                    if( enableDelayLoad )");
                            writer.WriteLine("                    {");
                            if (relation.RuntimeStyle == ORMRelationStyle.OneToManyWithKeyList
                                || relation.RuntimeStyle == ORMRelationStyle.OneToOne)
                            {
                                if (relation.ParentProperty == property)
                                {
                                    writer.WriteLine("                        ( ( " + typeName + "_Temp ) obj).DBValue_" + relation.ParentProperty.Name + " = reader.GetValue( iCount ) ;");
                                }
                            }
                            writer.WriteLine("                    }");
                            writer.WriteLine("                    else");
                            writer.WriteLine("                    {");
                            writer.WriteLine("                        if( relationValues != null )relationValues.Add( new RelationPropertyValue( instance , this.ORMType.Properties[\""
                                + property.Name + "\"] , reader.GetValue( iCount ) ));");
                            writer.WriteLine("                    }");
                        }
                        else
                        {
                            writer.WriteLine("                   if( relationValues != null )relationValues.Add( new RelationPropertyValue( instance , this.ORMType.Properties[\""
                                + property.Name + "\"] , reader.GetValue( iCount ) ));");
                        }
                    }
                    else
                    {
                        Type pt = property.PropertyType;
                        if (pt.IsEnum)
                        {
                            if ( referenceTypes != null 
                                && referenceTypes.Contains(pt) == false)
                            {
                                referenceTypes.Add(pt);
                            }
                            writer.WriteLine("                    obj." + property.Name 
                                + " = (" + ToTypeNameCode( pt , true ) + " ) DBValueToEnum( reader.GetValue( iCount ) , typeof( "
                                + ToTypeNameCode( pt ,true )+ " ));");
                        }
                        else
                        {
                            string ptTypeName = null;
                            if (pt.IsPrimitive 
                                || pt.Equals(typeof(string)) 
                                || pt.Equals( typeof( decimal ))
                                || pt.Equals( typeof( DateTime )))
                            {
                                ptTypeName = pt.Name;
                            }
                            else if (pt.Equals(typeof(byte[])))
                            {
                                ptTypeName = "Binary";
                            }
                            if (property.PropertyType.Equals(typeof(byte[])))
                            {
                                ptTypeName = "Binary";
                            }
                            if (ptTypeName == null)
                            {
                                throw new NotSupportedException( fullTypeName + "." + property.Name);
                            }
                            writer.WriteLine("                    obj." + property.Name
                                + " = DBValueTo" + ptTypeName
                                + " ( reader.GetValue( iCount ) , "
                                + ToStringValueCode(property.Format) + " );");
                        }
                    }
                }//foreach

                writer.WriteLine("                }");
                writer.WriteLine("                break;");
            }//foreach
            writer.WriteLine("            }//switch");
            writer.WriteLine("        }//for");
            writer.WriteLine("");
            writer.WriteLine("        return true;");
            writer.WriteLine("    }//");

            writer.WriteLine("");
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override string GetPropertyCommandText(string propertyName)");
            writer.WriteLine("    {");
            writer.WriteLine("        if (propertyName == null || propertyName.Trim().Length == 0)");
            writer.WriteLine("        {");
            writer.WriteLine("            return null;");
            writer.WriteLine("        }");
            writer.WriteLine("        switch (propertyName.Trim().ToLower())");
            writer.WriteLine("        {");
            foreach (ORMPropertyInfo property in type.Properties)
            {
                string strSQL =  helper.GetPropertyCommandText( property.Name );
                if (Util.HasContent(strSQL))
                {
                    writer.WriteLine("       case \"" + property.Name.ToLower() + "\": return " + ToStringValueCode(strSQL) + " ;");
                }
            }
            writer.WriteLine("        }");
            writer.WriteLine("        return null;");
            writer.WriteLine("    }");
            writer.WriteLine("");
            
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override object GetDBValue(IDataReader reader, string propertyName)");
            writer.WriteLine("    {");
            writer.WriteLine("        if (propertyName == null || propertyName.Trim().Length == 0)");
            writer.WriteLine("        {");
            writer.WriteLine("            return DBNull.Value;");
            writer.WriteLine("        }");
            writer.WriteLine("        switch (propertyName.Trim().ToLower())");
            writer.WriteLine("        {");
            foreach (ORMPropertyInfo property in type.Properties)
            {
                if (property.HasRelation == false)
                {
                    writer.WriteLine("            case  \"" + property.Name.Trim().ToLower()
                        + "\": return InnerGetDBValue( reader , "
                        + ToStringValueCode(property.RuntimeDBFieldName) + " , "
                        + ToStringValueCode(property.ComparabilityDBFieldName) + " , "
                        + ToStringValueCode(property.ComparabilityDBFieldName2) + " );");
                }//if
            }//foreach
            writer.WriteLine("        }");
            writer.WriteLine("        return DBNull.Value;");
            writer.WriteLine("    }");
            writer.WriteLine("");
            
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override object GenerateID(ORMContext context)");
            writer.WriteLine("    {");
            if (type.KeyProperty == null 
                || type.KeyProperty.IDGenerate == null)
            {
                writer.WriteLine("     throw new NotSupportedException( " + ToStringValueCode( fullTypeName + ".GenerateID") + ");");
            }
            else
            {
                ORMPropertyInfo kp = type.KeyProperty;
                switch (kp.IDGenerate.Style)
                {
                    case ORMIDGenerateStyle.CurrentTime:
                        if (Util.HasContent(kp.IDGenerate.Format))
                        {
                            writer.WriteLine("      return DateTime.Now.ToString( " + ToStringValueCode(kp.IDGenerate.Format) + ");");
                        }
                        else
                        {
                            writer.WriteLine("      return DateTime.Now ;");
                        }
                        break;
                    case ORMIDGenerateStyle.Custom:
                        break;
                    case ORMIDGenerateStyle.GUID:
                        if (Util.HasContent(kp.IDGenerate.Format))
                        {
                            writer.WriteLine("      return Guid.NewGuid().ToString( " + ToStringValueCode(kp.IDGenerate.Format) + ");");
                        }
                        else
                        {
                            if( kp.PropertyType.Equals( typeof(string )))
                            {
                                writer.WriteLine("      return Guid.NewGuid().ToString() ;");
                            }
                            else
                            {
                                writer.WriteLine("      return Guid.NewGuid() ;");
                            }
                        } break;
                    case ORMIDGenerateStyle.Increase:
                        writer.WriteLine("         return context.Framework.GenerateIncreaseID( context, "
                            + ToStringValueCode(type.TableName) + " , "
                            + ToStringValueCode(kp.Name) + " );");
                        break;
                }//switch
            }//else 

            writer.WriteLine("    }");
            writer.WriteLine("");
            
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override object GetKeyValue(object instance)");
            writer.WriteLine("    {");
            if (type.KeyProperty == null)
            {
                writer.WriteLine("     throw new NotSupportedException( "
                    + ToStringValueCode( fullTypeName + ".GetKeyValue") + ");");
            }
            else
            {
                writer.WriteLine("     return (( " + fullTypeName + ") instance)." + type.KeyProperty.Name + ";");
            }
            writer.WriteLine("    }");
            writer.WriteLine("");
            
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override object GetPropertyValue(object instance, string propertyName)");
            writer.WriteLine("    {");
            writer.WriteLine("        if (propertyName == null || propertyName.Trim().Length == 0)");
            writer.WriteLine("        {");
            writer.WriteLine("            throw new ArgumentNullException(\"propertyName\");");
            writer.WriteLine("        }");
            writer.WriteLine("        if (instance == null)");
            writer.WriteLine("        {");
            writer.WriteLine("            throw new ArgumentNullException(\"instance\");");
            writer.WriteLine("        }");
            writer.WriteLine("        " + fullTypeName + " obj = (" + fullTypeName + ")instance;");
            writer.WriteLine("        switch (propertyName.Trim().ToLower())");
            writer.WriteLine("        {");
            foreach (ORMPropertyInfo property in type.Properties)
            {
                writer.WriteLine("            case \"" + property.Name.ToLower() + "\" : return obj." + property.Name + ";");
            }
            writer.WriteLine("            default:");
            writer.WriteLine("                throw new NotSupportedException(propertyName);");
            writer.WriteLine("        }");
            writer.WriteLine("    }");
            writer.WriteLine("");
            
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override void SetPropertyValue(object instance, string propertyName, object Value)");
            writer.WriteLine("    {");
            writer.WriteLine("        if (propertyName == null || propertyName.Trim().Length == 0)");
            writer.WriteLine("        {");
            writer.WriteLine("            throw new ArgumentNullException(\"propertyName\");");
            writer.WriteLine("        }");
            writer.WriteLine("        if (instance == null)");
            writer.WriteLine("        {");
            writer.WriteLine("            throw new ArgumentNullException(\"instance\");");
            writer.WriteLine("        }");
            writer.WriteLine("        " + fullTypeName + " obj = (" + fullTypeName + ")instance;");
            writer.WriteLine("        switch (propertyName.Trim().ToLower())");
            writer.WriteLine("        {");
            foreach (ORMPropertyInfo property in type.Properties)
            {
                Type pt = property.PropertyType;
                string functionName = null;
                if (pt.Equals(typeof(bool)))
                {
                    functionName = "Convert.ToBoolean( Value )";
                }
                else if (pt.Equals(typeof(byte)))
                {
                    functionName = "Convert.ToByte( Value )";
                }
                else if (pt.Equals(typeof(char)))
                {
                    functionName = "Convert.ToChar( Value )" ;
                }
                else if( pt.Equals( typeof( DateTime )))
                {
                    functionName = "Convert.ToDateTime( Value )";
                }
                else if (pt.Equals(typeof(decimal)))
                {
                    functionName = "Convert.ToDecimal( Value )";
                }
                else if (pt.Equals(typeof(double)))
                {
                    functionName = "Convert.ToDouble( Value )";
                }
                else if (pt.IsEnum)
                {
                    functionName = "(" + ToTypeNameCode(pt , true ) + ")Convert.ToInt32( Value )";
                }
                else if (pt.Equals(typeof(Guid)))
                {
                    functionName = "(System.Guid) Value ";
                }
                else if (pt.Equals(typeof(short)))
                {
                    functionName = "Convert.ToInt16( Value )";
                }
                else if (pt.Equals(typeof(int)))
                {
                    functionName = "Convert.ToInt32( Value )";
                }
                else if (pt.Equals(typeof(long)))
                {
                    functionName = "Convert.ToInt64( Value )";
                }
                else if( pt.Equals( typeof( sbyte )))
                {
                    functionName = "Convert.ToSByte( Value )";
                }
                else if( pt.Equals( typeof( float )))
                {
                    functionName = "Convert.ToSingle( Value )";
                }
                else if( pt.Equals( typeof( string )))
                {
                    functionName = "Convert.ToString( Value )";
                }
                else if( pt.Equals( typeof( ushort )))
                {
                    functionName = "Convert.ToUInt16( Value)";
                }
                else if( pt.Equals( typeof( uint )))
                {
                    functionName = "Convert.ToUInt32( Value )";
                }
                else if( pt.Equals( typeof( ulong )))
                {
                    functionName = "Convert.ToUInt64( Value )";
                }
                else if( pt.Equals( typeof( byte[] )))
                {
                    functionName = "( byte[] ) Value";
                }
                else
                {
                    functionName = "( " + ToTypeNameCode( pt , true ) + " ) Value ";
                }
                writer.WriteLine("          case \"" + property.Name.ToLower() + "\":");
                writer.WriteLine("                obj." + property.Name + " = " + functionName + ";");
                writer.WriteLine("                break;");
            }//foreach
            writer.WriteLine("            default:");
            writer.WriteLine("                throw new NotSupportedException(propertyName);");
            writer.WriteLine("        }");
            writer.WriteLine("    }");
            writer.WriteLine("");
            writer.WriteLine("");


            ORMDBCommand tempCommand = new ORMDBCommand();
            context.Command = tempCommand;
            context.ForGenerateCode = true;
            
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override bool PrepareDeleteCommand(ORMContext context, object instance)");
            writer.WriteLine("    {");
            writer.WriteLine("        " + fullTypeName + " obj = (" + fullTypeName + ")instance;");
            writer.WriteLine("        context.Command.Parameters.Clear();");
            tempCommand.Parameters.Clear();
            helper.PrepareDeleteCommand( context , null );
            WriteCommandCode( writer , tempCommand , "context.Command", "obj" , null );
            writer.WriteLine("        return true;");
            writer.WriteLine("    }");
            writer.WriteLine("");
            
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override bool PrepareInsertCommand(ORMContext context, object instance)");
            writer.WriteLine("    {");
            writer.WriteLine("        " + fullTypeName + " obj = (" + fullTypeName + ")instance;");
            writer.WriteLine("        context.Command.Parameters.Clear();");
            tempCommand.Parameters.Clear();
            helper.PrepareInsertCommand( context , null );
            WriteCommandCode( writer , tempCommand , "context.Command", "obj" , null );
            writer.WriteLine("        return true;");
            writer.WriteLine("    }");
            writer.WriteLine("");

            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override bool PrepareUpdateCommand(ORMContext context, object instance)");
            writer.WriteLine("    {");
            writer.WriteLine("        " + fullTypeName + " obj = ( " + fullTypeName + ")instance;");
            writer.WriteLine("        context.Command.Parameters.Clear();");

            context.ForGenerateCode = true ;
            helper.PrepareUpdateCommand( context , null );
            WriteCommandCode( writer , tempCommand , "context.Command", "obj" , null );
            writer.WriteLine("        return true;");
            writer.WriteLine("    }");
            writer.WriteLine("");
            
            writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
            writer.WriteLine("    public override string SelectCommandText");
            writer.WriteLine("    {");
            writer.WriteLine("        get");
            writer.WriteLine("        {");
            writer.WriteLine("            return " + ToStringValueCode( helper.SelectCommandText ) + ";");
            writer.WriteLine("        }");
            writer.WriteLine("    }");
            writer.WriteLine("");

            if (generateTempClass)
            {
                writer.WriteLine("////////////////////////////////////////////////////////////////////////////////");
                writer.WriteLine("    [Serializable()]");
                writer.WriteLine("    public class " + typeName + "_Temp : " + ToTypeNameCode(type.InstanceType, true));
                writer.WriteLine("    {");
                writer.WriteLine("        public " + typeName + "_Temp()");
                writer.WriteLine("        {");
                writer.WriteLine("        }");
                writer.WriteLine("");
                writer.WriteLine("        [System.ComponentModel.Browsable(false)]");
                writer.WriteLine("        [System.Xml.Serialization.XmlIgnore()]");
                writer.WriteLine("        [NonSerialized()]");
                writer.WriteLine("        internal ORMEngine _Work = null;");
                writer.WriteLine("");

                foreach (ORMPropertyInfo property in type.Properties)
                {
                    if (property.DelayLoad)
                    {
                        writer.WriteLine("");
                        ORMRelationInfo relation = property.Relation;
                        Type pt = property.PropertyType;
                        bool generateTempValue = false;
                        if (relation != null)
                        {
                            if (relation.RuntimeStyle == ORMRelationStyle.OneToManyWithKeyList
                                || relation.RuntimeStyle == ORMRelationStyle.OneToOne)
                            {
                                if (relation.ParentProperty == property)
                                {
                                    writer.WriteLine("        internal object DBValue_" + property.Name + " = null;");
                                    generateTempValue = true;
                                }
                            }
                        }
                        writer.WriteLine("        /// <summary>");
                        writer.WriteLine("        /// Handle delay load");
                        writer.WriteLine("        /// </summary>");
                        writer.WriteLine("        [System.ComponentModel.EditorBrowsable(System.ComponentModel.EditorBrowsableState.Never)]");
                        writer.WriteLine("        public override " + ToTypeNameCode(pt, true) + " " + property.Name);
                        writer.WriteLine("        {");
                        writer.WriteLine("            get");
                        writer.WriteLine("            {");
                        if (generateTempValue)
                        {
                            writer.WriteLine("                if (base."
                                + property.Name + " == null && this._Work != null && this.DBValue_"
                                + property.Name + " != null && DBNull.Value.Equals( this.DBValue_"
                                + property.Name + " ) == false )");
                        }
                        else
                        {
                            writer.WriteLine("                if (base." + property.Name + " == null && this._Work != null )");
                        }
                        writer.WriteLine("                {");

                        string strCustomCommandText = property.CommandText;
                        if (relation != null && relation.RuntimeStyle == ORMRelationStyle.OneToMany)
                        {
                            Type itemType = CollectionTypeHelper.GetElementType(pt);
                            writer.WriteLine("");
                            writer.WriteLine("                    " + ToTypeNameCode(itemType, true) + " conditionInstance = new " + ToTypeNameCode(itemType, true) + "();");
                            if (relation.ChildProperty != null)
                            {
                                if (relation.ChildProperty.PropertyType.Equals( type.InstanceType ) )
                                {
                                    writer.WriteLine("                    conditionInstance." + relation.ChildProperty.Name + " = this;");
                                }
                                else
                                {
                                    writer.WriteLine("                    conditionInstance." + relation.ChildProperty.Name + " = this." + relation.ParentProperty.Name + " ;");
                                }
                            }
                            else
                            {
                                throw new NotSupportedException(fullTypeName + "." + property.Name + " Delay load");
                            }
                            writer.WriteLine("");
                            writer.WriteLine("                    Array list = this._Work.ReadInstancesByCondition(");
                            writer.WriteLine("                        conditionInstance,");
                            writer.WriteLine("                        new string[] { " + ToStringValueCode(relation.ChildProperty.Name) + " });");
                            writer.WriteLine("");
                            writer.WriteLine("                    " + ToTypeNameCode(pt, true) + " result = new " + ToTypeNameCode(pt, true) + "();");
                            writer.WriteLine("");
                            writer.WriteLine("                    foreach (" + ToTypeNameCode(itemType, true) + " item in list)");
                            writer.WriteLine("                    {");
                            writer.WriteLine("                        result.Add(item);");
                            writer.WriteLine("                    }");
                            writer.WriteLine("");
                            writer.WriteLine("                    base." + property.Name + " = result;");
                        }
                        else if (relation != null && relation.RuntimeStyle == ORMRelationStyle.OneToManyWithKeyList)
                        {
                            Type itemType = CollectionTypeHelper.GetElementType(pt);
                            string commandText = helper.GetPropertyCommandText(property.Name);
                            helper.FillDBCommand(context, commandText, null, property.Name, property);
                            writer.WriteLine("                    " + typeof(ORMDBCommand).FullName + " command = new " + typeof(ORMDBCommand).FullName + "();");
                            WriteCommandCode(writer, context.Command, "command", "this", property.Name);
                            writer.WriteLine("                    Array list = this._Work.ReadInstances( command , typeof( " + ToTypeNameCode(itemType, true) + " ));");
                            writer.WriteLine("                    " + ToTypeNameCode(pt, true) + " result = new " + ToTypeNameCode(pt, true) + "();");
                            writer.WriteLine("");
                            writer.WriteLine("                    foreach (" + ToTypeNameCode(itemType, true) + " item in list)");
                            writer.WriteLine("                    {");
                            writer.WriteLine("                        result.Add(item);");
                            writer.WriteLine("                    }");
                            writer.WriteLine("");
                            writer.WriteLine("                    base." + property.Name + " = result;");
                        }
                        else if (relation != null && relation.RuntimeStyle == ORMRelationStyle.OneToOne)
                        {
                            writer.WriteLine("                    base." + property.Name + " = ( " + ToTypeNameCode(property.PropertyType, true) + " )this._Work.ReadInstance( typeof( " + ToTypeNameCode(property.PropertyType, true) + ") , this.DBValue_" + property.Name + ");");
                        }
                        writer.WriteLine("                }");
                        writer.WriteLine("                return base." + property.Name + ";");
                        writer.WriteLine("            }");
                        writer.WriteLine("            set");
                        writer.WriteLine("            {");
                        writer.WriteLine("                base." + property.Name + " = value;");
                        if (relation.ParentProperty == property)
                        {
                            writer.WriteLine("                this.DBValue_" + property.Name + " = DBNull.Value ;");
                        }
                        writer.WriteLine("            }");
                        writer.WriteLine("        }");
                    }
                    
                }
                //writer.WriteLine("    public override string ToString(){ return base.ToString();}");

                writer.WriteLine("    }//public class TempClass");
            }
            writer.WriteLine("");
            writer.WriteLine("}//public class Helper");


            return writer.ToString();
        }
         
        private string AnalyseTemplates(Hashtable templateCodes , string nameSpace , string typeName )
        {
            
            StringBuilder myStr = new StringBuilder();
            myStr.AppendLine("using System;");
            myStr.AppendLine("using System.Collections ;");
            myStr.AppendLine("using System.Text;");
            myStr.AppendLine("namespace " + nameSpace);
            myStr.AppendLine("{");
            myStr.AppendLine("  public class " + typeName);
            myStr.AppendLine("  {");

            foreach (string methodName in templateCodes.Keys)
            {
                myStr.AppendLine("public static " + methodName + "( )");
                myStr.AppendLine("{");
                string code = AnalyseTemplate( ( string )templateCodes[methodName]);
                myStr.AppendLine(code);
                myStr.AppendLine("}");
            }

            myStr.AppendLine("  }");
            myStr.AppendLine("}");

            return myStr.ToString();
        }

        private string AnalyseTemplate(string templateCode)
        {
            if (templateCode == null)
            {
                return null;
            }
            string[] items = AnalyseVariableString(templateCode, "<%", "%>", true);
            StringBuilder myStr = new StringBuilder();
            for (int iCount = 0; iCount < items.Length; iCount++)
            {
                if ((iCount % 2) == 0)
                {
                    string item = items[ iCount ];
                    myStr.AppendLine();
                    myStr.AppendLine("Writer.Write(\"");
                    foreach (char c in item)
                    {
                        if( c == '\\' )
                        {
                            myStr.Append("\\\\");
                        }
                        else if (c == '\"')
                        {
                            myStr.Append("\\\"");
                        }
                        else if (c == '\r')
                        {
                            myStr.Append("\\\r");
                        }
                        else if (c == '\n')
                        {
                            myStr.Append("\\\n");
                        }
                        else
                        {
                            myStr.Append(c);
                        }
                    }//foreach
                    myStr.AppendLine("\";");
                }
                else
                {
                    string item = items[iCount];
                    if (item.StartsWith("@"))
                    {
                        item = item.Substring(1).Trim();
                    }
                    else
                    {
                        if (item.Trim().StartsWith("="))
                        {
                            myStr.AppendLine("Writer.Write(" + item + ");");
                        }
                        else
                        {
                            myStr.Append(item);
                        }
                    }
                }//else
            }//for
            return myStr.ToString();
        }

        /// <summary>
        /// 分析一个字符串,根据开始字符串和结束字符串进行分割,并返回分割生成的字符串数组
        /// 这些字符串数组的偶数号元素为开始字符串和结束字符串间的部分
        /// 例如分析字符串"aaa[bbb]ccc"产生字符串数组"aaa","bbb","ccc"
        /// </summary>
        /// <param name="strText">供分析的字符串</param>
        /// <param name="strHead">开始字符串</param>
        /// <param name="strEnd">结束字符串</param>
        /// <param name="EnableEmptyItem">允许出现空白项目</param>
        /// <returns>生成的字符串数组</returns>
        public static string[] AnalyseVariableString(
            string strText,
            string strHead,
            string strEnd,
            bool EnableEmptyItem)
        {
            // 若原始字符串无效或者没有任何可用的参数则退出函数
            if (strText == null
                || strHead == null
                || strEnd == null
                || strHead.Length == 0
                || strEnd.Length == 0
                || strText.Length == 0)
                return new string[] { strText };

            int index = strText.IndexOf(strHead);
            // 若原始字符串没有变量标记则退出函数
            if (index < 0)
                return new string[] { strText };

            System.Collections.ArrayList myList = new System.Collections.ArrayList();
            string strKey;
            int index2;
            int LastIndex = 0;
            do
            {
                // 查找有 "[内容]" 样式的子字符串
                // 若没有找到 "[" 和 "]"的字符对则退出循环
                index2 = strText.IndexOf(strEnd, index + 1);
                if (index2 > index)
                {
                    // 在 ] 前面查找最近的 [ 符号
                    index = LastIndexOfRange(strText, strHead, LastIndex, index2);
                    if (index == -1)
                    {
                        break;
                    }

                    // 获得字符对夹着的子字符串,该子字符串为参数名
                    // 若该参数名有效则向输出结果输出参数值
                    // 否则不进行额外的处理
                    int len = index2 - index - strHead.Length;
                    if (len == 0 && EnableEmptyItem == false)
                    {
                        break;
                    }

                    strKey = strText.Substring(index + strHead.Length, len);
                    if (LastIndex < index)
                    {
                        string strItem = strText.Substring(LastIndex, index - LastIndex);
                        myList.Add(strItem);
                    }
                    else
                        myList.Add("");
                    myList.Add(strKey);
                    index = index2 + strEnd.Length;
                    LastIndex = index;
                }
                else
                {
                    break;
                }
            } while (index >= 0 && index < strText.Length);
            // 添加处理过后剩余的字符串
            if (LastIndex < strText.Length)
                myList.Add(strText.Substring(LastIndex));
            return (string[])myList.ToArray(typeof(string));
        }//public static string[] AnalyseVariableString( string strText , string strHead , string strEnd )

        private static int LastIndexOfRange(
            string strText,
            string strSearch,
            int StartIndex,
            int EndIndex)
        {
            if (strText == null)
                throw new ArgumentNullException("strText");
            if (strSearch == null)
                throw new ArgumentNullException("strSearch");

            int LastIndex = strText.IndexOf(strSearch, StartIndex);
            if (LastIndex == -1)
                return -1;
            StartIndex += strSearch.Length;
            while (true)
            {
                int index = strText.IndexOf(strSearch, StartIndex);
                if (index == -1 || index >= EndIndex)
                {
                    return LastIndex;
                }
                LastIndex = index;
                StartIndex += strSearch.Length;
            }
            //return -1;
        }

    }

    
}
