﻿using System;
using System.Collections.Generic;
using System.Text;
using System.ComponentModel;

namespace XDesigner.ORM.DOM
{
    /// <summary>
    /// ORM relation information
    /// </summary>
    [Serializable()]
    public class ORMRelationInfo : ICloneable
    {
        /// <summary>
        /// Initialize instance
        /// </summary>
        public ORMRelationInfo()
        {
        }

        private ORMPropertyInfo _OwnerProperty = null;
        [Browsable( false )]
        [System.Xml.Serialization.XmlIgnore()]
        public ORMPropertyInfo OwnerProperty
        {
            get { return _OwnerProperty; }
            set { _OwnerProperty = value; }
        }

        private ORMRelationStyle _Style = ORMRelationStyle.Auto;
        /// <summary>
        /// Relation style
        /// </summary>
        public ORMRelationStyle Style
        {
            get { return _Style; }
            set { _Style = value; }
        }

        private ORMRelationStyle _RuntimeStyle = ORMRelationStyle.None;
        /// <summary>
        /// Relation style at runtime
        /// </summary>
        [System.Xml.Serialization.XmlIgnore()]
        [Browsable( false )]
        public ORMRelationStyle RuntimeStyle
        {
            get 
            {
                return _RuntimeStyle; 
            }
            set
            {
                _RuntimeStyle = value; 
            }
        }

        private string _ParentPropertyName = null;
        /// <summary>
        /// Parent field name
        /// </summary>
        public string ParentPropertyName
        {
            get { return _ParentPropertyName; }
            set
            {
                _ParentPropertyName = value;
            }
        }

        internal ORMPropertyInfo ParentProperty
        {
            get
            {
                if (this.OwnerProperty == null)
                    return null;
                if (Util.HasContent(this.ParentPropertyName))
                {
                    ORMPropertyInfo p = this.OwnerProperty.OwnerType.Properties.GetByName(
                        this.ParentPropertyName);
                    if (p == null)
                    {
                        if( this.RuntimeStyle == ORMRelationStyle.OneToOne )
                        {
                            p = this.OwnerProperty ;
                        }
                        else
                        {
                            p = this.OwnerProperty.OwnerType.KeyProperty;
                        }
                    }
                    return p;
                }
                else
                {
                    if (this.RuntimeStyle == ORMRelationStyle.OneToOne)
                    {
                        return this.OwnerProperty;
                    }
                    else
                    {
                        return this.OwnerProperty.OwnerType.KeyProperty;
                    }
                }
            }
        }
        
        private string _ChildPropertyName = null;
        /// <summary>
        /// Child field name
        /// </summary>
        public string ChildPropertyName
        {
            get { return _ChildPropertyName; }
            set { _ChildPropertyName = value; }
        }

        private ORMPropertyInfo _ChildProperty = null;

        internal ORMPropertyInfo ChildProperty
        {
            get { return _ChildProperty; }
            set { _ChildProperty = value; }
        }
        //{
        //    get
        //    {
        //        if (this.OwnerProperty != null)
        //        {
        //            ORMTypeInfo type = this.OwnerProperty.PropertyType .OwnerType;
        //            ORMPropertyInfo p = type.Properties.GetByName(this.ChildPropertyName);
        //            if (p == null)
        //            {
        //                p = type.KeyProperty;
        //            }
        //            return p;
        //        }
        //        else
        //        {
        //            return null;
        //        }
        //    }
        //}

        //private string _CustomCommandText = null;
        ///// <summary>
        ///// Custom command text for query data
        ///// </summary>
        //public string CustomCommandText
        //{
        //    get
        //    {
        //        return _CustomCommandText;
        //    }
        //    set
        //    {
        //        _CustomCommandText = value;
        //    }
        //}

        private bool _UniteDelete = false;
        /// <summary>
        /// When delete parent record , then delete child records
        /// </summary>
        [System.ComponentModel.DefaultValue(false )]
        public bool UniteDelete
        {
            get { return _UniteDelete; }
            set { _UniteDelete = value; }
        }

        object ICloneable.Clone()
        {
            ORMRelationInfo rel = new ORMRelationInfo();
            rel._ChildPropertyName = this._ChildPropertyName;
            rel._ParentPropertyName = this._ParentPropertyName;
            rel._Style = this._Style;
            rel._UniteDelete = this._UniteDelete;
            //rel.ParentProperty = this.ParentProperty;
            //rel.ChildProperty = this.ChildProperty;

            return rel;
        }

        public ORMRelationInfo Clone()
        {
            return (ORMRelationInfo)((ICloneable)this).Clone();
        }

    }

    [Serializable()]
    public class ORMRelationList : System.Collections.CollectionBase
    {
        public ORMRelationList()
        {
        }

        public ORMRelationInfo this[int index]
        {
            get
            {
                return (ORMRelationInfo)this.List[index];
            }
        }

        public int Add(ORMRelationInfo item)
        {
            return this.List.Add(item);
        }

        public void Remove(ORMRelationInfo item)
        {
            this.List.Remove(item);
        }
    }
}
