﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms ;
using XDesigner.ORM;
using XDesigner.ORM.DOM;
using System.Collections ;
using System.ComponentModel;
using System.Data;

namespace XDesigner.ORM
{
    public class ORMBindingSource : BindingSource
    {
        public ORMBindingSource()
        {
        }

        public ORMBindingSource(ORMEngine work, ORMTypeInfo type)
        {
            this._Framework = work;
            this._ORMType = type;
            if (work != null)
            {
                this._Helper = work.GetHelper(type);
            }
        }

        public ORMBindingSource(
            ORMEngine work,
            ORMTypeInfo type,
            System.Collections.IEnumerable instances)
        {
            this._Framework = work;
            this._ORMType = type;
            if (work != null)
            {
                this._Helper = work.GetHelper(type);
            }
            if (instances != null)
            {
                foreach (object obj in instances)
                {
                    this.Add(obj);
                }
            }
        }

        private ORMEngine _Framework = null;

        public ORMEngine Framework
        {
            get { return _Framework; }
            set { _Framework = value; }
        }
        private ORMTypeInfo _ORMType = null;

        public ORMTypeInfo ORMType
        {
            get { return _ORMType; }
            set { _ORMType = value; }
        }

        private ORMTypeMappingHelper _Helper = null;

        public ORMTypeMappingHelper Helper
        {
            get { return _Helper; }
            set { _Helper = value; }
        }

        private void CheckORMType( bool checkFramework )
        {
            if (_ORMType == null)
            {
                throw new InvalidOperationException("ORMType is null");
            }
            if (checkFramework)
            {
                if (_Framework == null)
                {
                    throw new InvalidOperationException("Framework is null");
                }
            }
        }

        private bool _LogItemChange = false;
        /// <summary>
        /// can log item change task.
        /// </summary>
        public bool LogItemChange
        {
            get { return _LogItemChange; }
            set { _LogItemChange = value; }
        }

        public override int Add(object value)
        {
            CheckORMType(false);
            this.ORMType.CheckInstanceType(value, true);
            return base.Add(value);
        }

        public void AddRange(System.Collections.IEnumerable items)
        {
            CheckORMType(false);
            foreach (object item in items)
            {
                this.ORMType.CheckInstanceType(item, true);
                base.Add(item);
            }
        }

        protected override void OnAddingNew(System.ComponentModel.AddingNewEventArgs e)
        {
            base.OnAddingNew(e);
            if (e.NewObject == null)
            {
                CheckORMType(true);
                this.Framework.OpenConnection();
                ORMTypeMappingHelper helper = this.Framework.GetHelper(this.ORMType);
                object instance = helper.CreateInstance();
                this.Framework.GenerateInstaceID(instance);
                e.NewObject = instance;
            }
        }

        public override void ApplySort(ListSortDescriptionCollection sorts)
        {
            // sort items by bubble.
            int itemCount = this.Count;

            for (int iCount1 = 0; iCount1 < itemCount; iCount1++)
            {
                for (int iCount2 = itemCount - 1; iCount2 > iCount1; iCount2--)
                {
                    object item1 = this[iCount2 - 1];
                    object item2 = this[iCount2];
                    foreach (ListSortDescription sort in sorts)
                    {
                        int result = CompareItem(
                            item1, 
                            item2, 
                            sort.PropertyDescriptor,
                            sort.SortDirection);
                        if (result > 0)
                        {
                            this[iCount1] = item2;
                            this[iCount2] = item1;
                            break;
                        }
                        else if (result < 0)
                        {
                            break;
                        }
                    }
                }//for
            }//for
        }

        public override void ApplySort(PropertyDescriptor property, ListSortDirection sort)
        {
            // sort items by bubble.
            int itemCount = this.Count;
            for (int iCount1 = 0; iCount1 < itemCount; iCount1++)
            {
                for( int iCount2 = itemCount -1 ; iCount2 > iCount1 ; iCount2 -- )
                {
                    object item1 = this[iCount2 - 1];
                    object item2 = this[iCount2];
                    int result = CompareItem(item1, item2, property, sort);
                    if (result > 0)
                    {
                        this[iCount1] = item2;
                        this[iCount2] = item1;
                    }
                }//for
            }//for
        }


        protected virtual int CompareItem(
            object item1,
            object item2 ,
            PropertyDescriptor property,
            ListSortDirection sort)
        {
            object v1 = null;
            object v2 = null;
            if ( this._Helper == null)
            {
                v1 = property.GetValue(item1);
                v2 = property.GetValue(item2);
            }
            else
            {
                v1 = this._Helper.GetPropertyValue(item1, property.Name);
                v2 = this._Helper.GetPropertyValue(item2, property.Name);
            }
            int result = 0;
            if (v1 is System.IComparable)
            {
                result = ((IComparable)v1).CompareTo(v2);
            }
            if (sort == ListSortDirection.Descending)
            {
                result = -result;
            }
            return result;
        }

        //private Hashtable myRecordState = new Hashtable();

        public event EventHandler Changed = null;

        private Dictionary<object, DataRowState> myRecordStates
            = new Dictionary<object, DataRowState>();

        [System.ComponentModel.Browsable( false )]
        public bool Modified
        {
            get
            {
                return myRecordStates.Count > 0;
            }
        }

        public bool SetRecordState(object instance, DataRowState state)
        {
            if (this.Contains(instance))
            {
                if (myRecordStates.ContainsKey(instance)
                    && myRecordStates[instance] == DataRowState.Added)
                {
                    if (this.Changed != null)
                    {
                        this.Changed(this, EventArgs.Empty);
                    }
                }
                else
                {
                    myRecordStates[instance] = state;
                    if (this.Changed != null)
                    {
                        this.Changed(this, EventArgs.Empty);
                    }
                }
                return true;
            }
            else
            {
                return false;
            }
        }

        public override void Remove(object value)
        {
            if (this.LogItemChange)
            {
                LogRemoveInstance(value);
            }
            base.Remove(value);
        }

        public override void RemoveAt(int index)
        {
            if (this.LogItemChange)
            {
                object instance = this.List[ index ] ;
                LogRemoveInstance(instance);
            }
            base.RemoveAt(index);
        }

        private void LogRemoveInstance(object instance)
        {
            if (myRecordStates.ContainsKey(instance))
            {
                DataRowState state = myRecordStates[instance];
                if (state == System.Data.DataRowState.Added)
                {
                    myRecordStates.Remove(instance);
                }
                else
                {
                    myRecordStates[instance] = DataRowState.Deleted;
                }
            }
            else
            {
                myRecordStates[instance] = DataRowState.Deleted ;
            }
            if (this.Changed != null)
            {
                this.Changed(this, EventArgs.Empty);
            }
        }

        
        protected override void OnListChanged( ListChangedEventArgs e)
        {
            if (this.LogItemChange)
            {
                switch (e.ListChangedType)
                {
                    case ListChangedType.ItemAdded:
                        {
                            myRecordStates[this.List[e.NewIndex]] = System.Data.DataRowState.Added;
                            if (this.Changed != null)
                            {
                                this.Changed(this, EventArgs.Empty);
                            }
                        }
                        break;
                    case ListChangedType.ItemChanged:
                        {
                            object instance = this.List[e.NewIndex];
                            if (myRecordStates.ContainsKey(instance) == false )
                            {
                                myRecordStates[instance] = DataRowState.Modified;
                            }
                            if (this.Changed != null)
                            {
                                this.Changed(this, EventArgs.Empty);
                            }
                        }
                        break;
                }
            }
            base.OnListChanged(e);
        }
         

        /// <summary>
        /// Update data to database
        /// </summary>
        /// <returns>number of database records effect</returns>
        public int Update( )
        {
            int result = 0;
            if (myRecordStates.Count > 0)
            {
                this.Framework.OpenConnection();
                foreach (object instance in myRecordStates.Keys)
                {
                    DataRowState state = myRecordStates[instance];
                    if (state == System.Data.DataRowState.Added)
                    {
                        result += this.Framework.Insert(instance);
                    }
                    else if (state == System.Data.DataRowState.Deleted)
                    {
                        result += this.Framework.Delete(instance);
                    }
                    else if (state == System.Data.DataRowState.Modified)
                    {
                        result += this.Framework.Update(instance);
                    }
                }//foreach
                myRecordStates.Clear();
            }
            return result;
        }//public int Update( )

        protected override void Dispose(bool disposing)
        {
            if (this.Changed != null )
            {
                this.Changed = null;
            }
            base.Dispose(disposing);
        }
    }
}
