﻿using System;
using System.Collections.Generic;
using System.Text;
using XDesigner.ORM.DOM ;
using System.Collections;

namespace XDesigner.ORM
{
    public class ORMEventSender
    {
        public ORMEventSender()
        {
        }

        private Dictionary<object, ORMEventHandler> _Handlers
            = new Dictionary<object, ORMEventHandler>();

        public void AddListener(object key, ORMEventHandler handler)
        {
            if (key == null)
            {
                throw new ArgumentNullException("key");
            }
            if (handler == null)
            {
                throw new ArgumentNullException("handler");
            }
            this._Handlers[key] = handler;
        }

        public void RemoveListener(object key)
        {
            if (key == null)
            {
                throw new ArgumentNullException("key");
            }
            if ( this._Handlers.ContainsKey(key))
            {
                this._Handlers.Remove(key);
            }
        }

        public void Clear()
        {
            this._Handlers.Clear();
        }

        public void Raise(object sender, ORMEventArgs args)
        {
            ArrayList list = new ArrayList();
            list.AddRange(_Handlers.Values);
            if ( this._Handlers.ContainsKey(sender))
            {
                list.Remove( this._Handlers[sender]);
            }
            foreach (ORMEventHandler handler in list)
            {
                handler(sender, args);
            }
        }

    }//public class ORMEventSender


    public delegate void ORMEventHandler( object sender , ORMEventArgs args );

    /// <summary>
    /// ORM event arguments
    /// </summary>
    public class ORMEventArgs : EventArgs
    {
        //public ORMEventArgs()
        //{
        //}

        public ORMEventArgs(ORMEngine work, ORMEventStyle style)
        {
            _Framework = work;
            _Style = style;
        }

        public ORMEventArgs(ORMContext context, ORMEventStyle style, ORMTypeInfo type)
        {
            _Context = context;
            if (_Context != null)
            {
                _Framework = context.Framework;
            }
            _Style = style;
            _ORMType = type;
        }

        private ORMEngine _Framework = null;

        public ORMEngine Framework
        {
            get { return _Framework; }
        }

        private ORMContext _Context = null;

        public ORMContext Context
        {
            get { return _Context; }
        }

        private ORMEventStyle _Style = ORMEventStyle.Read;

        public ORMEventStyle Style
        {
            get { return _Style; }
        }

        private ORMTypeInfo _ORMType = null;

        public ORMTypeInfo ORMType
        {
            get { return _ORMType; }
        }

        private bool _Cancel = false;
        /// <summary>
        /// Flag that user cancel event
        /// </summary>
        public bool Cancel
        {
            get { return _Cancel; }
            set { _Cancel = value; }
        }

        private object _Result = null;

        public object Result
        {
            get { return _Result; }
            set { _Result = value; }
        }
    }

    public enum ORMEventStyle
    {
        Read,
        /// <summary>
        /// Specify table is modified.
        /// </summary>
        Modified ,
        OpenConnection ,
        CloseConnection
    }
}
