﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Collections ;

namespace XDesigner.ORM
{
    public class ORMInstanceBuffer
    {
        public ORMInstanceBuffer()
        {
        }

        private Hashtable myBuffer = new Hashtable();

        private Hashtable GetTable(Type instanceType)
        {
            if (instanceType == null)
                return null;
            foreach (Type t in myBuffer.Keys)
            {
                if (instanceType.Equals(t) || t.IsSubclassOf(instanceType))
                {
                    return (Hashtable)myBuffer[t];
                }
            }
            return null;
        }

        public object GetBufferedInstance(object key, Type instanceType)
        {
            if (key == null)
            {
                return null;
            }
            if (instanceType == null)
            {
                throw new ArgumentNullException("instanceType");
            }
            Hashtable table = GetTable(instanceType);
            if (table != null)
            {
                string strKey = key.ToString();
                return table[strKey];
            }
            else
            {
                return null;
            }
        }

        public void SaveBufferedInstance(Type instanceType , object key, object instance)
        {
            if (key == null)
            {
                throw new ArgumentNullException("key");
            }
            if (instance == null)
            {
                throw new ArgumentNullException("instance");
            }
            Hashtable table = GetTable(instanceType);
            if (table == null)
            {
                table = new Hashtable();
                myBuffer[instanceType] = table;
            }
            table[key.ToString()] = instance;
        }

        public void RemoveBufferedInstance(object key, object instance)
        {
            if (key == null)
            {
                throw new ArgumentNullException("key");
            }
            if (instance == null)
            {
                throw new ArgumentNullException("instance");
            }
            Hashtable table = GetTable(instance.GetType());
            if (table != null)
            {
                string strKey = key.ToString();
                if (table.ContainsKey(strKey))
                {
                    table.Remove(strKey);
                }
            }
        }

        public void Clear()
        {
            myBuffer.Clear();
        }

        public void Clear(Type instanceType)
        {
            if (instanceType != null)
            {
                if (myBuffer.ContainsKey(instanceType))
                {
                    myBuffer.Remove(instanceType);
                }
            }
        }
    }
}
