﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection ;
using System.Collections ;

namespace XDesigner.ORM
{
    internal sealed class ORMReflectionHelper 
    {

        private static Hashtable myPropertyBindingInfos = new Hashtable();
        public static PropertyBindingInfo[] GetFieldProperties(Type instanceType)
        {
            if (myPropertyBindingInfos.ContainsKey(instanceType))
            {
                return (PropertyBindingInfo[])myPropertyBindingInfos[instanceType];
            }
            else
            {
                ArrayList list = new ArrayList();
                foreach (PropertyInfo property in instanceType.GetProperties(
                    BindingFlags.Public | BindingFlags.Instance))
                {
                    if (Attribute.GetCustomAttribute(property, typeof(ORMIgnoreAttribute), true) != null )
                    {
                        continue;
                    }
                    PropertyBindingInfo info = new PropertyBindingInfo();
                    info.Name = property.Name;
                    info.Property = property;
                    ORMFieldAttribute field = (ORMFieldAttribute)Attribute.GetCustomAttribute(property, typeof(ORMFieldAttribute), true);
                    if (field == null)
                    {
                        if (HasContent(field.FieldName))
                        {
                            info.Name = field.FieldName;
                        }
                    }
                    if (Attribute.GetCustomAttribute(property, typeof(ORMKeyFieldAttribute)) != null)
                    {
                        info.KeyField = true;
                    }
                    if (Attribute.GetCustomAttribute(property, typeof(ORMDelayAttribute)) != null)
                    {
                        info.Delay = true;
                    }
                    System.ComponentModel.DefaultValueAttribute dv = (System.ComponentModel.DefaultValueAttribute)Attribute.GetCustomAttribute( property , typeof(System.ComponentModel.DefaultValueAttribute));
                    if (dv != null)
                    {
                        info.DefaultValue = dv.Value;
                    }
                    list.Add(info);
                }//foreach
                PropertyBindingInfo[] infos = (PropertyBindingInfo[])list.ToArray(typeof(PropertyBindingInfo));
                myPropertyBindingInfos[instanceType] = infos;
                return infos;
            }
        }

        private static bool HasContent(string text)
        {
            return text != null && text.Trim().Length > 0;
        }
         
    }

    internal class PropertyBindingInfo
    {
        public string Name = null;
        public PropertyInfo Property = null;
        public bool KeyField = false;
        public bool Delay = false;
        public object DefaultValue = null;
    }

}
